@@ -5,7 +5,7 @@
 #include <limits.h>
 #include <linux/net.h>
 #include <sys/socket.h>
-#include <sys/un.h>
+#include <sys/types.h>
 
 #include "rr/rr.h"
 
@@ -181,8 +181,8 @@ void AutoRemoteSyscalls::restore_state_to(Task* t) {
   // Unmap our scatch region if required
   if (scratch_mem_was_mapped) {
     AutoRemoteSyscalls remote(t, DISABLE_MEMORY_PARAMS);
-    remote.infallible_syscall(syscall_number_for_munmap(arch()), fixed_sp,
-                              4096);
+    remote.infallible_syscall(syscall_number_for_munmap(arch()),
+                              fixed_sp - 4096, 4096);
   }
   if (!replaced_bytes.empty()) {
     t->write_mem(remote_ptr<uint8_t>(initial_regs.ip().to_data_ptr<uint8_t>()),
@@ -328,113 +328,6 @@ static remote_ptr<T> allocate(remote_ptr<void>* buf_end,
   return allocate(buf_end, remote_buf, sizeof(T)).cast<T>();
 }
 
-static int create_bind_and_listen_socket(const char* path) {
-  struct sockaddr_un addr;
-  int listen_sock = socket(AF_UNIX, SOCK_STREAM, 0);
-  if (listen_sock < 0) {
-    FATAL() << "Failed to create listen socket";
-  }
-
-  addr.sun_family = AF_UNIX;
-  strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
-  addr.sun_path[sizeof(addr.sun_path) - 1] = 0;
-  if (::bind(listen_sock, (struct sockaddr*)&addr, sizeof(addr))) {
-    FATAL() << "Failed to bind listen socket";
-  }
-
-  if (chmod(path, 0666)) {
-    FATAL() << "Failed to make listening socket world-writeable";
-  }
-
-  if (listen(listen_sock, 1)) {
-    FATAL() << "Failed to mark listening for listen socket";
-  }
-
-  return listen_sock;
-}
-
-template <typename Arch>
-static int child_create_socket(AutoRemoteSyscalls& remote,
-                               remote_ptr<socketcall_args<Arch>> sc_args) {
-  int child_sock;
-  if (sc_args.is_null()) {
-    child_sock =
-        remote.infallible_syscall(Arch::socket, AF_UNIX, SOCK_STREAM, 0);
-  } else {
-    write_socketcall_args<Arch>(remote.task(), sc_args, AF_UNIX, SOCK_STREAM,
-                                0);
-    child_sock =
-        remote.infallible_syscall(Arch::socketcall, SYS_SOCKET, sc_args);
-  }
-  return child_sock;
-}
-
-template <typename Arch>
-static int child_connect_socket(AutoRemoteSyscalls& remote,
-                                AutoRestoreMem& remote_buf,
-                                remote_ptr<socketcall_args<Arch>> sc_args,
-                                remote_ptr<void> buf_end, int child_sock,
-                                const char* path, int* cwd_fd) {
-  typename Arch::sockaddr_un addr;
-  memset(&addr, 0, sizeof(addr)); // Make valgrind happy.
-  addr.sun_family = AF_UNIX;
-  DEBUG_ASSERT(strlen(path) < sizeof(addr.sun_path));
-  // Skip leading '/' since we're going to access this relative to the root
-  DEBUG_ASSERT(path[0] == '/');
-  strcpy(addr.sun_path, path + 1);
-
-  auto tmp_buf_end = buf_end;
-  auto remote_dot = allocate(&tmp_buf_end, remote_buf, 2);
-  remote.task()->write_mem(remote_dot.cast<char>(), ".", 2);
-  // This may fail because the tracee has done a setuid and is no longer
-  // able to access its own CWD. There doesn't seem to be a way to save
-  // and restore the CWD in this situation --- and connectat() doesn't exist yet
-  // so we have no way to avoid having to set the CWD if we're in a chrooted
-  // environment. We'll hack around such a failure later by putting the child
-  // into a new directory that it can't access.
-  *cwd_fd = remote.syscall(syscall_number_for_open(Arch::arch()), remote_dot,
-                           O_PATH | O_DIRECTORY);
-  ASSERT(remote.task(), *cwd_fd >= 0 || *cwd_fd == -EACCES);
-  remote.infallible_syscall(Arch::fchdir, RR_RESERVED_ROOT_DIR_FD);
-
-  auto remote_addr = allocate<typename Arch::sockaddr_un>(&buf_end, remote_buf);
-  remote.task()->write_mem(remote_addr, addr);
-  if (sc_args.is_null()) {
-    return remote.syscall(Arch::connect, child_sock, remote_addr, sizeof(addr));
-  } else {
-    write_socketcall_args<Arch>(remote.task(), sc_args, child_sock,
-                                remote_addr.as_int(), sizeof(addr));
-    return remote.syscall(Arch::socketcall, SYS_CONNECT, sc_args);
-  }
-}
-
-// Restore tracee CWD. Its CWD is currently rr's root.
-template <typename Arch>
-static void restore_cwd(AutoRemoteSyscalls& remote, int cwd_fd) {
-  if (cwd_fd >= 0) {
-    remote.infallible_syscall(Arch::fchdir, cwd_fd);
-    remote.infallible_syscall(Arch::close, cwd_fd);
-  } else {
-    // We can't get back to the original directory. (Note that we may not
-    // be able to access it ourselves from outside the tracee, if it's in a
-    // mount namespace created by the tracee.)
-    // Fake it by making a world-accessible directory, chdir the tracee into it,
-    // then make it inaccessible.
-    char path[PATH_MAX];
-    snprintf(path, sizeof(path) - 1, "%s/rr-tracee-fd-inaccessible-dir-%d-%ld",
-             tmp_dir(), remote.task()->tid, random());
-    path[sizeof(path) - 1] = 0;
-    int ret = mkdir(path, 0777);
-    ASSERT(remote.task(), ret >= 0);
-    AutoRestoreMem mem(remote, path);
-    remote.infallible_syscall(Arch::chdir, mem.get());
-    ret = chmod(path, 0000);
-    ASSERT(remote.task(), ret >= 0);
-    ret = rmdir(path);
-    ASSERT(remote.task(), ret >= 0);
-  }
-}
-
 template <typename Arch>
 static long child_sendmsg(AutoRemoteSyscalls& remote,
                           AutoRestoreMem& remote_buf,
@@ -484,7 +377,7 @@ static long child_sendmsg(AutoRemoteSyscalls& remote,
   }
 }
 
-static int recvmsg_socket(int sock) {
+static int recvmsg_socket(ScopedFd& sock) {
   char received_data;
   struct iovec msgdata;
   msgdata.iov_base = &received_data;
@@ -512,12 +405,6 @@ static int recvmsg_socket(int sock) {
 
 template <typename T> static size_t reserve() { return align_size(sizeof(T)); }
 
-static void* do_accept(void* p) {
-  int fd = intptr_t(p);
-  int sock = accept(fd, nullptr, nullptr);
-  return (void*)intptr_t(sock);
-}
-
 template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
   size_t data_length = std::max(reserve<typename Arch::sockaddr_un>(),
                                 reserve<typename Arch::msghdr>() +
@@ -534,78 +421,12 @@ template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
     sc_args = allocate<socketcall_args<Arch>>(&sc_args_end, remote_buf);
   }
 
-  char path[PATH_MAX];
-  snprintf(path, sizeof(path) - 1, "%s/rr-tracee-fd-transfer-%d-%ld", tmp_dir(),
-           t->tid, random());
-  path[sizeof(path) - 1] = 0;
-
-  int listen_sock = create_bind_and_listen_socket(path);
-  int child_sock = child_create_socket(*this, sc_args);
-
-  // There's a tricky potential deadlock here. We need to puppet the tracee's
-  // connect() syscall and at the same time accept() it from our process; the
-  // tracee's connect() cannot complete until our accept() has happened.
-  // We could launch the tracee connect() syscall and then do a blocking
-  // accept() here before waiting for the connect() to complete, but then there
-  // is a risk that a rogue signal could interrupt the tracee's connect(),
-  // putting it in a ptrace-stop which we never detect because our accept()
-  // never finishes.
-  // A simple safe way to avoid the deadlock is to create our own temporary
-  // thread to do the accept(). That lets us treat the connect() call like
-  // any other blocking tracee AutoRemoteSyscall. If this were to become a
-  // performance problem, the best solution would probably to be to keep a
-  // dedicated accept-thread around.
-
-  pthread_t accept_thread;
-  int ret = pthread_create(&accept_thread, nullptr, do_accept,
-                           (void*)intptr_t(listen_sock));
-  ASSERT(t, ret >= 0) << "Cannot create helper thread";
-
-  // This changes the CWD of the tracee to rr's root (via RR_RESERVED_ROOT_FD).
-  // The old cwd is available in cwd_fd except in bizarre situations where the
-  // child cannot open its own CWD, in which case cwd_fd is -1.
-  int cwd_fd;
-  int child_connect = child_connect_socket(
-      *this, remote_buf, sc_args, sc_args_end, child_sock, path, &cwd_fd);
-  ASSERT(t, child_connect == 0)
-      << "Failed to connect child socket, err=" << errno_name(-child_connect);
-
-  void* thread_ret;
-  ret = pthread_join(accept_thread, &thread_ret);
-  ASSERT(t, ret == 0) << "Cannot join helper thread";
-  int sock = intptr_t(thread_ret);
-  ASSERT(t, sock >= 0) << "Failed to create parent socket";
-
-  // Verify that the child's PID is correct. This is a world-writeable socket
-  // so anyone could connect, and this check is vital.
-  struct ucred cred;
-  cred.pid = -1;
-  socklen_t credlen = sizeof(cred);
-  if (getsockopt(sock, SOL_SOCKET, SO_PEERCRED, &cred, &credlen) < 0) {
-    ASSERT(t, false) << "Failed SO_PEERCRED";
-  }
-  ASSERT(t, credlen == sizeof(cred));
-  if (cred.pid != t->real_tgid()) {
-    FATAL() << "Some other process connected to our socket!!!";
-  }
-
-  // Restore the child's CWD to what it used to be.
-  restore_cwd<Arch>(*this, cwd_fd);
-
-  // Listening socket not needed anymore
-  close(listen_sock);
-  unlink(path);
   long child_syscall_result =
-      child_sendmsg(*this, remote_buf, sc_args, sc_args_end, child_sock, fd);
+      child_sendmsg(*this, remote_buf, sc_args, sc_args_end,
+                    task()->session().tracee_fd_number(), fd);
   ASSERT(t, child_syscall_result > 0) << "Failed to sendmsg() in tracee; err="
                                       << errno_name(-child_syscall_result);
-  // Child may be waiting on our recvmsg().
-  int our_fd = recvmsg_socket(sock);
-
-  child_syscall_result = infallible_syscall(Arch::close, child_sock);
-  ret = close(sock);
-  ASSERT(t, ret == 0) << "Failed to close parent socket";
-
+  int our_fd = recvmsg_socket(task()->session().tracee_socket_fd());
   return ScopedFd(our_fd);
 }
 
