@@ -5,7 +5,9 @@
 
 #include <stdint.h>
 
-class Task;
+namespace rr {
+
+class ReplayTask;
 
 /**
  * Helper to detect when the "CPUID can cause rcbs to be lost" bug is present.
@@ -31,12 +33,7 @@ public:
   /**
    * Call this when task t enters a traced syscall during replay.
    */
-  void notify_reached_syscall_during_replay(Task* t);
-  /**
-   * Returns true when the "CPUID can cause rcbs to be lost" bug has
-   * been detected.
-   */
-  bool is_cpuid_bug_detected() { return detected_cpuid_bug; }
+  void notify_reached_syscall_during_replay(ReplayTask* t);
 
 private:
   uint64_t trace_rcb_count_at_last_geteuid32;
@@ -44,4 +41,6 @@ private:
   bool detected_cpuid_bug;
 };
 
+} // namespace rr
+
 #endif /* RR_CPUID_BUG_DETECTOR_H_ */
