@@ -13,6 +13,8 @@
 
 namespace rr {
 
+struct XSaveLayout;
+
 /**
  * An ExtraRegisters object contains values for all user-space-visible
  * registers other than those in Registers.
@@ -47,23 +49,16 @@ public:
    * currently save and restore but do not otherwise use. If the data record
    * has more than 512 bytes then it's an XSAVE(64) area, otherwise it's just
    * the FXSAVE(64) area.
+   *
+   * The data always uses our CPU's native XSAVE layout. When reading a trace,
+   * we need to convert from the trace's CPU's XSAVE layout to our layout.
    */
   enum Format { NONE, XSAVE };
 
-  // Set values from raw data
-  void set_to_raw_data(SupportedArch a, Format format,
-                       std::vector<uint8_t>& consume_data) {
-    arch_ = a;
-    format_ = format;
-    std::swap(data_, consume_data);
-
-    // The actual XSAVE minimum size is 576 bytes (512 followed by a 64-byte
-    // XSAVE header) but sometimes we're storing an FXSAVE(64) which is only
-    // 512 bytes.
-    static const size_t min_xsave_size = 512;
-    assert(format == NONE || data_.size() >= min_xsave_size);
-  }
-
+  // Set values from raw data, with the given XSAVE layout. Returns false
+  // if this could not be done.
+  bool set_to_raw_data(SupportedArch a, Format format, const uint8_t* data,
+                       size_t data_size, const XSaveLayout& layout);
   Format format() const { return format_; }
   SupportedArch arch() const { return arch_; }
   const std::vector<uint8_t> data() const { return data_; }
@@ -71,6 +66,11 @@ public:
   const uint8_t* data_bytes() const { return data_.data(); }
   bool empty() const { return data_.empty(); }
 
+  /**
+   * Read XSAVE `xinuse` field
+   */
+  uint64_t read_xinuse(bool* defined) const;
+
   /**
    * Like |Registers::read_register()|, except attempts to read
    * the value of an "extra register" (floating point / vector).
@@ -85,7 +85,8 @@ public:
   /**
    * Update registers from a user_fpregs_struct.
    */
-  void set_user_fpregs_struct(SupportedArch arch, void* data, size_t size);
+  void set_user_fpregs_struct(Task* t, SupportedArch arch, void* data,
+                              size_t size);
 
   /**
    * Get a user_fpxregs_struct for from these ExtraRegisters.
@@ -95,7 +96,8 @@ public:
   /**
    * Update registers from a user_fpxregs_struct.
    */
-  void set_user_fpxregs_struct(const X86Arch::user_fpxregs_struct& regs);
+  void set_user_fpxregs_struct(Task* t,
+                               const X86Arch::user_fpxregs_struct& regs);
 
   void print_register_file_compact(FILE* f) const;
 
