@@ -53,7 +53,7 @@ void MmappedFileMonitor::did_write(Task* t, const std::vector<Range>& ranges,
 
   bool is_replay = t->session().is_replaying();
   for (auto v : t->session().vms()) {
-    for (auto m : v->maps()) {
+    for (const auto& m : v->maps()) {
       auto km = m.map;
 
       if (is_replay) {
@@ -85,9 +85,14 @@ void MmappedFileMonitor::did_write(Task* t, const std::vector<Range>& ranges,
             // If we're writing beyond the EmuFile's end, resize it.
             m.emu_file->ensure_size(local_offset + r.length);
           } else {
+            ASSERT(t, !v->task_set().empty());
+            Task* tt = *v->task_set().begin();
             // We will record multiple writes if the file is mapped multiple
-            // times. This is inefficient, but not wrong.
-            static_cast<RecordTask*>(t)->record_remote(km.intersect(mr));
+            // times. This is inefficient --- one is sufficient --- but not
+            // wrong.
+            // Make sure we use a task for this address space. `t` might have
+            // a different address space.
+            static_cast<RecordTask*>(tt)->record_remote(km.intersect(mr));
           }
         }
         local_offset += r.length;
