@@ -319,20 +319,19 @@ void ReplayTimeline::seek_to_before_key(const MarkKey& key) {
     } else {
       // Return one of the checkpoints at *it.
       current = nullptr;
-      for (auto mark_it : marks[it->first]) {
-        shared_ptr<InternalMark> m(mark_it);
-        if (m->checkpoint) {
-          current = m->checkpoint->clone();
-          // At this point, m->checkpoint is fully initialized but current
-          // is not. Swap them so that m->checkpoint is not fully
+      for (const auto& mark_it : marks[it->first]) {
+        if (mark_it->checkpoint) {
+          current = mark_it->checkpoint->clone();
+          // At this point, mark_it->checkpoint is fully initialized but current
+          // is not. Swap them so that mark_it->checkpoint is not fully
           // initialized, to reduce resource usage.
-          swap(current, m->checkpoint);
+          swap(current, mark_it->checkpoint);
+          breakpoints_applied = false;
+          current_at_or_after_mark = mark_it;
           break;
         }
       }
       DEBUG_ASSERT(current);
-      breakpoints_applied = false;
-      current_at_or_after_mark = nullptr;
     }
   }
 }
@@ -551,7 +550,7 @@ void ReplayTimeline::seek_to_mark(const Mark& mark) {
 bool ReplayTimeline::fix_watchpoint_coalescing_quirk(ReplayResult& result,
                                                      const ProtoMark& before) {
   if (result.status == REPLAY_EXITED ||
-      result.break_status.watchpoints_hit.empty()) {
+      result.break_status.data_watchpoints_hit().empty()) {
     // no watchpoint hit. Nothing to fix.
     return false;
   }
@@ -580,7 +579,7 @@ bool ReplayTimeline::fix_watchpoint_coalescing_quirk(ReplayResult& result,
         // if a watchpoint is triggered by the string instruction at
         // string_instruction_ip, it will have the correct timing.
         result = current->replay_step(RUN_SINGLESTEP_FAST_FORWARD);
-        if (!result.break_status.watchpoints_hit.empty()) {
+        if (!result.break_status.data_watchpoints_hit().empty()) {
           LOG(debug) << "Fixed x86-string coalescing quirk; now at "
                      << current_mark_key() << " (new cx "
                      << result.break_status.task->regs().cx() << ")";
@@ -688,6 +687,12 @@ void ReplayTimeline::apply_breakpoints_internal() {
       vm->add_breakpoint(get<1>(bp), BKPT_USER);
     }
   }
+  for (auto& wp : watchpoints) {
+    AddressSpace* vm = current->find_address_space(get<0>(wp));
+    if (vm && get<3>(wp) == WATCH_EXEC) {
+      vm->add_watchpoint(get<1>(wp), get<2>(wp), get<3>(wp));
+    }
+  }
 }
 
 void ReplayTimeline::apply_breakpoints_and_watchpoints() {
@@ -703,7 +708,7 @@ void ReplayTimeline::apply_breakpoints_and_watchpoints() {
     // be created) and we should reapply watchpoints then.
     // XXX we could make this more efficient by providing a method to set
     // several watchpoints at once on a given AddressSpace.
-    if (vm) {
+    if (vm && get<3>(wp) != WATCH_EXEC) {
       vm->add_watchpoint(get<1>(wp), get<2>(wp), get<3>(wp));
     }
   }
@@ -715,6 +720,12 @@ void ReplayTimeline::unapply_breakpoints_internal() {
     if (vm) {
       vm->remove_breakpoint(get<1>(bp), BKPT_USER);
     }
+    for (auto& wp : watchpoints) {
+      AddressSpace* vm = current->find_address_space(get<0>(wp));
+      if (vm && get<3>(wp) == WATCH_EXEC) {
+        vm->remove_watchpoint(get<1>(wp), get<2>(wp), get<3>(wp));
+      }
+    }
   }
 }
 
@@ -726,7 +737,7 @@ void ReplayTimeline::unapply_breakpoints_and_watchpoints() {
   unapply_breakpoints_internal();
   for (auto& wp : watchpoints) {
     AddressSpace* vm = current->find_address_space(get<0>(wp));
-    if (vm) {
+    if (vm && get<3>(wp) != WATCH_EXEC) {
       vm->remove_watchpoint(get<1>(wp), get<2>(wp), get<3>(wp));
     }
   }
@@ -928,7 +939,7 @@ ReplayResult ReplayTimeline::reverse_continue(
         // populate the interval with new checkpoints, speeding up
         // the following seek and possibly future operations.
       }
-      at_breakpoint = result.break_status.breakpoint_hit;
+      at_breakpoint = result.break_status.hardware_or_software_breakpoint_hit();
       bool avoidable_stop = result.break_status.breakpoint_hit ||
                             !result.break_status.watchpoints_hit.empty();
       if (avoidable_stop) {
@@ -952,14 +963,14 @@ ReplayResult ReplayTimeline::reverse_continue(
         set_short_checkpoint();
       }
 
-      if (!result.break_status.watchpoints_hit.empty() ||
+      if (!result.break_status.data_watchpoints_hit().empty() ||
           result.break_status.signal) {
         dest = mark();
         if (result.break_status.signal) {
           LOG(debug) << "Found signal break at " << dest;
         } else {
           LOG(debug) << "Found watch break at " << dest << ", addr="
-                     << result.break_status.watchpoints_hit[0].addr;
+                     << result.break_status.data_watchpoints_hit()[0].addr;
         }
         final_result = result;
         final_tuid = result.break_status.task ? result.break_status.task->tuid()
@@ -989,7 +1000,7 @@ ReplayResult ReplayTimeline::reverse_continue(
 
       // If there is a breakpoint at the current ip() where we start a
       // reverse-continue, gdb expects us to skip it.
-      if (result.break_status.breakpoint_hit) {
+      if (result.break_status.hardware_or_software_breakpoint_hit()) {
         dest = mark();
         LOG(debug) << "Found breakpoint break at " << dest;
         final_result = result;
@@ -1168,6 +1179,9 @@ ReplayResult ReplayTimeline::reverse_singlestep(
     Mark step_start = set_short_checkpoint();
     ReplayResult destination_candidate_result;
     TaskUid destination_candidate_tuid;
+    // True when the singlestep starting at the destination candidate saw
+    // another task break.
+    bool destination_candidate_saw_other_task_break = false;
 
     if (is_start_of_reverse_execution_barrier_event()) {
       destination_candidate = mark();
@@ -1189,7 +1203,7 @@ ReplayResult ReplayTimeline::reverse_singlestep(
           constraints.stop_before_states.push_back(&end.ptr->proto.regs);
           result = current->replay_step(constraints);
           update_observable_break_status(now, result);
-          if (result.break_status.breakpoint_hit) {
+          if (result.break_status.hardware_or_software_breakpoint_hit()) {
             // If we hit a breakpoint while singlestepping, we didn't
             // make any progress.
             unapply_breakpoints_and_watchpoints();
@@ -1208,6 +1222,7 @@ ReplayResult ReplayTimeline::reverse_singlestep(
                        << destination_candidate;
             destination_candidate_result = result;
             destination_candidate_tuid = result.break_status.task->tuid();
+            destination_candidate_saw_other_task_break = seen_other_task_break;
             seen_other_task_break = false;
             step_start = now;
           }
@@ -1217,7 +1232,7 @@ ReplayResult ReplayTimeline::reverse_singlestep(
           if (result.break_status.any_break()) {
             seen_other_task_break = true;
           }
-          if (result.break_status.breakpoint_hit) {
+          if (result.break_status.hardware_or_software_breakpoint_hit()) {
             unapply_breakpoints_and_watchpoints();
             result = current->replay_step(RUN_SINGLESTEP_FAST_FORWARD);
             update_observable_break_status(now, result);
@@ -1239,6 +1254,7 @@ ReplayResult ReplayTimeline::reverse_singlestep(
         destination_candidate_result = result;
         destination_candidate_result.break_status.task_exit = true;
         destination_candidate_tuid = current->current_task()->tuid();
+        destination_candidate_saw_other_task_break = false;
         seen_other_task_break = false;
       }
 
@@ -1251,7 +1267,7 @@ ReplayResult ReplayTimeline::reverse_singlestep(
     no_watchpoints_hit_interval_end =
         no_watchpoints_hit_interval_start ? end : Mark();
 
-    if (seen_other_task_break) {
+    if (seen_other_task_break || destination_candidate_saw_other_task_break) {
       // We saw a break in another task that the debugger cares about, but
       // that's not the stepping task. Report that break instead of the
       // singlestep.
@@ -1345,7 +1361,8 @@ ReplayResult ReplayTimeline::replay_step_forward(RunCommand command,
   }
   maybe_add_reverse_exec_checkpoint(LOW_OVERHEAD);
 
-  bool did_hit_breakpoint = result.break_status.breakpoint_hit;
+  bool did_hit_breakpoint =
+      result.break_status.hardware_or_software_breakpoint_hit();
   evaluate_conditions(result);
   if (did_hit_breakpoint && !result.break_status.any_break()) {
     // Singlestep past the breakpoint
