@@ -70,8 +70,8 @@ struct BreakStatus {
   std::vector<WatchConfig> watchpoints_hit;
   // When non-null, we stopped because a signal was delivered to |task|.
   std::unique_ptr<siginfo_t> signal;
-  // True when we stopped because we hit a breakpoint at |task|'s current
-  // ip().
+  // True when we stopped because we hit a software breakpoint at |task|'s
+  // current ip().
   bool breakpoint_hit;
   // True when we stopped because a singlestep completed in |task|.
   bool singlestep_complete;
@@ -81,6 +81,29 @@ struct BreakStatus {
   // True when we stopped because |task| is about to exit.
   bool task_exit;
 
+  // True when we stopped because we hit a software or hardware breakpoint at
+  // |task|'s current ip().
+  bool hardware_or_software_breakpoint_hit() {
+    for (const auto& w : watchpoints_hit) {
+      // Hardware execution watchpoints behave like breakpoints: the CPU
+      // stops before the instruction is executed.
+      if (w.type == WATCH_EXEC) {
+        return true;
+      }
+    }
+    return breakpoint_hit;
+  }
+  // Returns just the data watchpoints hit.
+  std::vector<WatchConfig> data_watchpoints_hit() {
+    std::vector<WatchConfig> result;
+    for (const auto& w : watchpoints_hit) {
+      if (w.type != WATCH_EXEC) {
+        result.push_back(w);
+      }
+    }
+    return result;
+  }
+
   bool any_break() {
     return !watchpoints_hit.empty() || signal || breakpoint_hit ||
            singlestep_complete || approaching_ticks_target;
@@ -159,8 +182,10 @@ public:
    */
   std::shared_ptr<AddressSpace> clone(Task* t,
                                       std::shared_ptr<AddressSpace> vm);
-
-  std::shared_ptr<ThreadGroup> create_tg(Task* t);
+  /**
+   * Create the initial thread group.
+   */
+  std::shared_ptr<ThreadGroup> create_initial_tg(Task* t);
   /**
    * Return a copy of |tg| with the same mappings.
    */
@@ -298,6 +323,9 @@ public:
 
   static const char* rr_mapping_prefix();
 
+  ScopedFd& tracee_socket_fd() { return *tracee_socket; }
+  int tracee_fd_number() const { return tracee_socket_fd_number; }
+
 protected:
   Session();
   virtual ~Session();
@@ -331,6 +359,8 @@ protected:
 
   Statistics statistics_;
 
+  std::shared_ptr<ScopedFd> tracee_socket;
+  int tracee_socket_fd_number;
   uint32_t next_task_serial_;
   ScopedFd spawned_task_error_fd_;
 
