@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* Test for the 'flock' system call. */
 
-#include "rrutil.h"
+#include "util.h"
 
 #define FILENAME "foo.txt"
 
