@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static void check_mapping(int* rpage, int* wpage, ssize_t nr_ints) {
   int i;
@@ -17,6 +17,7 @@ static void check_mapping(int* rpage, int* wpage, ssize_t nr_ints) {
 static void overwrite_file(const char* path, ssize_t num_bytes) {
   const int magic = 0x5a5a5a5a;
   int fd = open(path, O_TRUNC | O_RDWR, 0600);
+  test_assert(fd >= 0);
   size_t i;
   for (i = 0; i < num_bytes / sizeof(magic); ++i) {
     write(fd, &magic, sizeof(magic));
@@ -24,10 +25,11 @@ static void overwrite_file(const char* path, ssize_t num_bytes) {
   close(fd);
 }
 
+static const char file_name[] = "tmpfile";
+
 int main(void) {
   size_t num_bytes = sysconf(_SC_PAGESIZE);
-  char file_name[] = "/tmp/rr-test-mremap-XXXXXX";
-  int fd = mkstemp(file_name);
+  int fd = open(file_name, O_CREAT | O_EXCL | O_RDWR, 0600);
   int* wpage;
   int* rpage;
   int* old_wpage;
@@ -61,5 +63,7 @@ int main(void) {
 
   atomic_puts("EXIT-SUCCESS");
 
+  unlink(file_name);
+
   return 0;
 }
