@@ -1,11 +1,11 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "nsutils.h"
-#include "rrutil.h"
+#include "util.h"
 
 int main(void) {
   if (-1 == try_setup_ns(CLONE_NEWNS)) {
-    atomic_printf("EXIT-SUCCESS");
+    atomic_puts("EXIT-SUCCESS");
     return 0;
   }
 
@@ -39,6 +39,6 @@ int main(void) {
   test_assert(0 == stat("new_root.txt", &buf));
   test_assert(0 == stat("new_old_root/old_root.txt", &buf));
 
-  atomic_printf("EXIT-SUCCESS");
+  atomic_puts("EXIT-SUCCESS");
   return 0;
 }
