@@ -95,6 +95,9 @@ static void init_device(int fd) {
     buf->mmap_data = mmap(NULL, buf->vbuf.length, PROT_READ | PROT_WRITE,
                           MAP_SHARED, fd, buf->vbuf.m.offset);
     test_assert(buf->mmap_data != MAP_FAILED);
+    atomic_printf("Buffer %d, addr %p, device offset 0x%llx, device len 0x%llx\n",
+                  (int)i, buf->mmap_data, (long long)buf->vbuf.m.offset,
+                  (long long)buf->vbuf.length);
     test_assert(0 == ioctl(fd, VIDIOC_QBUF, &buf->vbuf));
   }
   type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -184,6 +187,7 @@ static void read_frames(int fd) {
     struct v4l2_buffer buf;
     int ret;
     size_t bytes;
+    struct buffer* buffer;
 
     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     buf.memory = V4L2_MEMORY_MMAP;
@@ -192,11 +196,22 @@ static void read_frames(int fd) {
     test_assert(buf.index < buffer_count);
 
     bytes = buf.length < 16 ? buf.length : 16;
-    atomic_printf("Frame %d: buffer %d: ", (int)i, (int)buf.index);
+    buffer = &buffers[buf.index];
+    atomic_printf("Frame %d, buffer %d, addr %p: ", (int)i, (int)buf.index,
+                  buffer->mmap_data);
     for (j = 0; j < bytes; ++j) {
-      atomic_printf("%2x ", buffers[buf.index].mmap_data[j]);
+      atomic_printf("%2x ", buffer->mmap_data[j]);
     }
     atomic_printf("...\n");
+
+    /* Reallocate the mmap data to check for bugs involving the length
+       of the shared memory area */
+    munmap(buffer->mmap_data, buffer->vbuf.length);
+    buffer->mmap_data =
+      mmap(NULL, buffer->vbuf.length, PROT_READ | PROT_WRITE, MAP_SHARED,
+           fd, buffer->vbuf.m.offset);
+    test_assert(buffer->mmap_data != MAP_FAILED);
+
     test_assert(0 == ioctl(fd, VIDIOC_QBUF, &buf));
   }
 }
