@@ -159,6 +159,7 @@ enum cpuid_requests {
 const int OSXSAVE_FEATURE_FLAG = 1 << 27;
 const int AVX_FEATURE_FLAG = 1 << 28;
 const int HLE_FEATURE_FLAG = 1 << 4;
+const int XSAVEC_FEATURE_FLAG = 1 << 1;
 
 /** issue a single request to CPUID. Fits 'intel features', for instance
  *  note that even if only "eax" and "edx" are of interest, other registers
@@ -171,6 +172,15 @@ struct CPUIDData {
 };
 CPUIDData cpuid(uint32_t code, uint32_t subrequest);
 
+/**
+ * Check OSXSAVE flag.
+ */
+bool xsave_enabled();
+/**
+ * Fetch current XCR0 value using XGETBV instruction.
+ */
+uint64_t xcr0();
+
 /**
  * Return all CPUID values supported by this CPU.
  */
@@ -370,7 +380,7 @@ std::vector<std::string> current_env();
 
 int get_num_cpus();
 
-enum class DisabledInsn {
+enum class TrappedInstruction {
   NONE = 0,
   RDTSC = 1,
   RDTSCP = 2,
@@ -378,10 +388,10 @@ enum class DisabledInsn {
 };
 
 /* If |t->ip()| points at a disabled instruction, return the instruction */
-DisabledInsn disabled_insn_at(Task* t, remote_code_ptr ip);
+TrappedInstruction trapped_instruction_at(Task* t, remote_code_ptr ip);
 
-/* Return the length of the DisabledInsn */
-size_t disabled_insn_len(DisabledInsn insn);
+/* Return the length of the TrappedInstruction */
+size_t trapped_instruction_len(TrappedInstruction insn);
 
 /**
  * BIND_CPU means binding to a randomly chosen CPU.
